package explorviz.live_trace_processing.filter.reduction.language_based;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import explorviz.live_trace_processing.filter.IPipeReceiver;
import explorviz.live_trace_processing.filter.reduction.AbstractReductionFilter;
import explorviz.live_trace_processing.record.event.AbstractAfterEventRecord;
import explorviz.live_trace_processing.record.event.AbstractBeforeEventRecord;
import explorviz.live_trace_processing.record.event.AbstractEventRecord;
import explorviz.live_trace_processing.record.trace.Trace;

public abstract class AbstractLanguageBasedFilter extends AbstractReductionFilter {

	public AbstractLanguageBasedFilter(final IPipeReceiver receiver, final String counterString) {
		super(receiver, counterString);
	}

	@Override
	protected Trace reduceTrace(final Trace trace) {
		final List<AbstractEventRecord> traceEvents = new ArrayList<AbstractEventRecord>();
		final Stack<Integer> stack = new Stack<Integer>();
		int count = 0;
		for (final AbstractEventRecord event : trace.getTraceEvents()) {
			if (event instanceof AbstractBeforeEventRecord) {
				count++;
				final AbstractBeforeEventRecord abstractBeforeEventRecord = (AbstractBeforeEventRecord) event;
				if (isSoughtEvent(abstractBeforeEventRecord)) {
					stack.push(new Integer(count));
				} else {
					traceEvents.add(event);
				}
			} else if (event instanceof AbstractAfterEventRecord) {
				if (!stack.isEmpty() && (count == stack.peek())) {
					stack.pop();
				} else {
					traceEvents.add(event);
				}
				count--;
			}
		}
		if (traceEvents.size() > 0) {
			return new Trace(traceEvents, true);
		} else {
			return new Trace(traceEvents, false);
		}
	}

	protected abstract boolean isSoughtEvent(AbstractBeforeEventRecord abstractBeforeEventRecord);
}
